clear all
n = 10;
i=20000;                %iterations
dt=0.000005;            %initial time step
R=20;                   %radius of the circle
a=0.4;                  %open angle, a*pi
b=0.015/3;              %cutting limit of segment length
c=0.05/3;               %adding limit of segment length
L=0.0001;               %migration constant
M=10;                   %diffusion constant
g1=1;                   %bulk energy for phaseI
g2=1;                   %bulk energy for phaseI
r=5;                    %surface tension of object(droplet)
rr=3.5;                 %surface tension between droplet and solid support
E=0;                    %electric field
D=zeros(1,n);           %dipole

%function [X Y] = CircleCoordinates(R, n, a);       circle generation
%function [F] = formFmatrix (theta, l, r, g1, g2, E, D, n)  Cal & form F
%function [H] = formHmatrix (theta, l, L, M, n)             Cal & form H
%function [D A] = distAngle(X,Y) A:Angle D:Distence   Cal legnth and angle


[X Y] = CircleCoordinates(R, n, a);
figure(1);
for j=1:i;
    SX=zeros(1,n); %Speed of X
    SY=zeros(1,n); %Speed of X
    [l theta] = distAngle(X,Y);
    Angle(j) = theta(1)/pi*180;
    itr(j)=j;
    seg(j)=l(n);
%     if j==2000   %code used when recording movie to three parts
%     pause(15)    
%     end 
%     if j==8000
%     pause(15)    
%     end    


% plot main simulation figure: shape change of the droplet
    subplot(3,2,1:4), plot(X, Y, 'bo-'), title({'Simulation',[' Number of iterations: ', num2str(j)], [' Time step: ', num2str(dt,'%08.4g')]}),xlabel('X'),ylabel('Y')

    
% plot contact angle vs iterations. showing contact angle we have now and
% theoriotical contact angle at steady state; after 5000th iterations, we
% also add average contact angle from 5000th iteration to end.
    if j <= 5000 
        subplot(3,2,5), plot(itr, Angle, 'b-'), title({[' Contact Angle: ', num2str(theta(1)/pi*180,'%0.4g')], [' Theo Angle: ', num2str(acos(1-rr/r)/pi*180,'%0.4g')]}),xlabel('Interation'),ylabel('Angle(degree)')

    else
        avgAngle=sum(Angle(5001:end))/(j-5000);
        subplot(3,2,5), plot(itr, Angle, 'b-'), title({[' Contact Angle: ', num2str(theta(1)/pi*180,'%0.4g')], [' Average Angle: ', num2str(avgAngle,'%0.4g')], [' Theo Angle: ', num2str(acos(1-rr/r)/pi*180,'%0.4g')]}),xlabel('Iteration'),ylabel('Angle(degree)')

    end
    
% plot the legnth of bottom segment vs iteration    
    subplot(3,2,6), plot(itr, seg, 'b-'), title([' Length: ', num2str(l(n),'%0.4g')]),xlabel('Iteration'),ylabel('Length')

    
    pause(0.0000001)
    
% calculate H, F matrix, add boundary conditions
    [H] = formHmatrix (theta, l, L, M, n);
    [F] = formFmatrix (theta, l, r, g1, g2, E, D, n);

    F(1) = F(1) + rr;
    F(4*n-3) = F(4*n-3) -rr;

    H(2,2) = H(2,2)+ 100000; %y1=0
    H(4*n-2,4*n-2) = H(4*n-2,4*n-2)+ 100000; %y(n)=0
    


    for k=1:n;
        H(4*k-3,4*k-3)= H(4*k-3,4*k-3)+0.000001;
        H(4*k-2,4*k-2)= H(4*k-2,4*k-2)+0.000001;
    end
   
%get speed of points
    Rate = H\F;
    
    for k=1:(n);
        SX(k) = Rate(4*k-3);
        SY(k) = Rate(4*k-2);
    end

%check if time step is too large or too small, and bring it back to a
%controlled region
    Vmax = zeros(1,n);
    for q = 1:n
        Vmax(q) = sqrt(SX(q)^2 + SY(q)^2);
    end
    
     if dt >= 2*pi*R*b/max(Vmax)/2
         dt = 2*pi*R*b/max(Vmax)/4;
     elseif dt <= 2*pi*R*b/max(Vmax)/10
         dt = 2*pi*R*b/max(Vmax)/4;
     end

%update coordinates
    X = X + SX*dt;
    Y = Y + SY*dt;
    
%check if segement needs to be cut or point needs to be added
    lmax=n;
    w=1;
    
    while w < lmax
       if l(w) < (2*pi*R*b) %Cutting points
           if X(w) > 0
               X(w+1) = [];
               Y(w+1) = [];
               l(w+1) = [];
           elseif w ~= lmax
               X(w) = [];
               Y(w) = [];
               l(w) = [];
           end
           lmax = length(X);
%            display('Deleting a point...');  %code used to debug
%            output = [w lmax n length(l)];
%            disp(output)
       end
       w=w+1;
    end
    n = length(X);
    lmax = n;
    w = 1;
    while w < lmax
       if l(w) >(2*pi*R*c) %Adding points 
         Xtop = X(1:w);
         Xbottom = X(w+1:end);
         Xmid = (X(w) + X(w+1))/2;
         X = [Xtop Xmid Xbottom];

         Ytop = Y(1:w);
         Ybottom = Y(w+1:end);
         Ymid = (Y(w) + Y(w+1))/2;
         Y = [Ytop Ymid Ybottom];
                              
         ltop = l(1:w);
         ltop(w) = l(w)/2;
         lbottom = l(w+1:end);
         lmid = l(w)/2;
         l = vertcat(ltop, lmid, lbottom);
         lmax = length(X);      
%          display('Adding a point...');
         w = w + 1;
%          output = [w lmax n length(l)];
%          disp(output);
       end
       w=w+1;
    end
    n = length(X);
end

